/* -*- compile-command: "msdev tentacles.dsp /make \"Tentacles - Win32 Debug\"" -*-
 */

#include <modeler.h>
#include <math.h>

MODEL(Tentacles);

#define VIEW_X  1
#define VIEW_Y  2
#define VIEW_Z  3

#define TWIST   10
#define CURL    11
#define MANUAL  12
#define SPIN    13
#define NOD     14

void segment( double length, double radius );

#define MAX(x,y)  ( (x) < (y) ? (y) : (x) )

int Model_SetupCmd( ClientData clientdata, Tcl_Interp* interp,
		    int objc, Tcl_Obj* const objv[] )
{
    scale( interp, VIEW_X, "x view", 0.0, 360.0, 1.0, 280.0 );
    scale( interp, VIEW_Y, "y view", 0.0, 360.0, 1.0, 0.0 );
    scale( interp, VIEW_Z, "z view", 0.0, 360.0, 1.0, 35.0 );

    checkbox( interp, MANUAL, "manual control", 0 );
    
    scale( interp, TWIST, "twist", -45.0, 45.0, 0.5, 0.0 );
    scale( interp, CURL, "curl", -90.0, 90.0, 0.5, 0.0 );
    scale( interp, SPIN, "spin", 0.0, 360.0, 1.0, 0.0 );
    scale( interp, NOD, "nod", -60.0, 60.0, 0.5, 0.0 );
    
    return TCL_OK;
}

int Model_ResizeCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    int width, height;

    if ( objc != 3 )
    {
	Tcl_SetResult( interp, "wrong # args: should be \"width height\"",
		       TCL_STATIC );
	return TCL_ERROR;
    }

    if ( Tcl_GetIntFromObj( interp, objv[1], &width ) != TCL_OK )
	return TCL_ERROR;
    if ( Tcl_GetIntFromObj( interp, objv[2], &height ) != TCL_OK )
	return TCL_ERROR;

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
    gluPerspective( 30.0, (double)width/height, 1.0, 100.0 );
    glViewport( 0, 0, width, height );

    glClearColor( 0.0, 0.0, 0.0, 0.0 );

    return TCL_OK;
}

int Model_RedrawCmd( ClientData clientdata, Tcl_Interp* interp,
		     int objc, Tcl_Obj* const objv[] )
{
    int lighting;
    int i, j;
    double twist, curl, spin, nod;
    double length;
    double time;
    
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    draw_mode_setup();

    time = get_time();
    
    glTranslated( 0.0, -8.0, -35.0 );
    glRotated( get_control_d( VIEW_X ), 1.0, 0.0, 0.0 );
    glRotated( get_control_d( VIEW_Y ), 0.0, 1.0, 0.0 );
    glRotated( get_control_d( VIEW_Z ), 0.0, 0.0, 1.0 );

#if 0
    glGetIntegerv( GL_LIGHTING, &lighting );
    glDisable( GL_LIGHTING );
    glBegin( GL_LINES );
    glColor3d( 1.0, 0.0, 0.0 );
    glVertex3d( 0.0, 0.0, 0.0 );
    glVertex3d( 5.0, 0.0, 0.0 );
    glColor3d( 0.0, 1.0, 0.0 );
    glVertex3d( 0.0, 0.0, 0.0 );
    glVertex3d( 0.0, 5.0, 0.0 );
    glColor3d( 0.0, 0.0, 1.0 );
    glVertex3d( 0.0, 0.0, 0.0 );
    glVertex3d( 0.0, 0.0, 5.0 );
    glEnd();
    if ( lighting == GL_TRUE )
	glEnable( GL_LIGHTING );
#endif

    diffuse_color( 0.5, 0.5, 0.5 );

    if ( get_control_b( MANUAL ) )
    {
	twist = get_control_d( TWIST );
	curl = get_control_d( CURL );
	spin = get_control_d( SPIN );
	nod = get_control_d( NOD );
    }
    else
	spin = 360.0 * time / 15.0;
    
    diffuse_color( 0.0, 0.3, 0.8 );
    sphere( 1.0 );
    glRotated( spin, 0.0, 0.0, 1.0 );
    
    for ( j = 0; j < 5; ++j )
    {
	if ( ! get_control_b( MANUAL ) )
	{
	    twist = 45.0 * sin( MAX(time-j,0.0) / 10.0 * M_PI * 2.0 );
	    curl = 60.0 * sin( MAX(time-j,0.0) / 10.0 * M_PI * 4.0 / 7.0 );
	    nod = 30.0 * sin( MAX(time-j,0.0) / 10.0 * M_PI * 3.0 / 2.0 );
	}
    
	glRotated( 72.0, 0.0, 0.0, 1.0 );
	glPushMatrix();
	glRotated( nod, 1.0, 0.0, 0.0 );

	length = 2.0;
	
	for ( i = 0; i < 10; ++i )
	{
	    glPushMatrix();
	    segment( length, 0.25 );
	    glTranslated( 0.0, 0.0, length );
	    length *= 0.95;
	    
	    glRotated( twist, 0.0, 0.0, 1.0 );
	    glRotated( curl, 1.0, 0.0, 0.0 );
	}
	
	for ( i = 0; i < 10; ++i )
	    glPopMatrix();

	glPopMatrix();
    }

    return TCL_OK;
}

void segment( double length, double radius )
{
    diffuse_color( 0.5, 0.5, 0.5 );
    cylinder( length, radius, radius );
    glPushMatrix();
    glTranslated( 0.0, 0.0, length );
    diffuse_color( 0.0, 0.3, 0.8 );
    sphere( radius * 1.2 );
    glPopMatrix();
}

